/************************** * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2004 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
*************************** * * * * * * * * * * * * **************************/

#ifndef _QHACCUTILS_H
#define _QHACCUTILS_H

#include "qhaccconstants.h"

#include <iostream>
using namespace std;

#include <qobject.h>

class QHacc;

class Utils {
 public:
	~Utils();

	static Utils * get();
	static void parser( const QString& str, const QString& sep, int start,
											QString * tokens, int numT );
	static bool debug( int, std::ostream *& );
	static bool error( int, std::ostream *& );
	static void setDebug( int );
	static int getDebug();

	static const int PRINT, ERRFATAL, ERROPER, ERRDEFAULT, DBGMAJOR,
		DBGMINOR, CURIOSITY;

	static QDate dateFromString( const QString& date, const QString& sep,
															 int form );
	static QString stringFromDate( const QDate& date, const QString& sep,
																 int form );
	static QString shortStringFromDate( const QDate& date, const QString& sep,
																			int form );

	/**
	 * @return the name of the given field
	 */
	static const char * tcname( Table t, int field );

	/**
	 * @return the type of the given field
	 */
	static ColType tctype( Table t, int field );

	/**
	 * @return the field number for the given string
	 */
	static int tcnum( Table t, const QString& name );
	
	/**
	 * @return the number of columns in the given table
	 */
	static int tcols( Table t );

	/**
	 * @return the primary key column for given table
	 */
	static int tpk( Table t );

	/**
	 * @return the field names of the given table
	 */
	static const char ** tnames( Table t );

	/**
	 * @return the types of the given table
	 */
	static const ColType * ttypes( Table t );

 private:
	static const int COLS[];
	static const char ** NAMES[];
	static const ColType * TYPES[];
	static const int PKS[];
	
	static Utils * obj;
	static int QD;

	/* debug levels are:
	 *
	 * 0: no debugging nor errors
	 * 1: fatal errors only
	 * 2: errors that affect operation
	 * 3: errors requiring default values to be used
	 * 4: print major debugging information
	 * 5: print minor debugging information
	 * 6: for the curiosity cat
	 *
	 */
};

class MonCon : public QObject {
	Q_OBJECT
		public:

	MonCon( QHacc * engine );
	~MonCon();
	
	/**
	 * Convert the given sum, but leaves it in a state that the engine's
	 * convMoney function can understand it
	 * @param curretval A sum string in the engine's format (such as directly
	 * from a Split row).
	 * @returns the currentval * conversion factor, with no currency separator
	 * switching
	 */
	QString converte( const QString& currentval ) const;

	/**
	 * Convert the given sum, but leaves it in a state that the engine's
	 * convMoney function can understand it
	 * from a Split row).
	 * @returns the currentval * conversion factor, with no currency separator
	 * switching
	 */
	QString converte( int currentval ) const;

	/**
	 * Convert the given sum
	 * @returns the currentval * conversion factor
	 */
	QString convert( const QString& currentval ) const;

	/**
	 * Convert the given sum
	 * @returns the currentval * conversion factor
	 */
	QString convert( int currentval ) const;

	/**
	 * Convert the given sum
	 * @returns the currentval/conversion factor
	 */	
	QString unconvert( const QString& currentval ) const;

	/**
	 * Convert the given sum
	 * @returns the currentval/conversion factor
	 */
	QString unconvert( int currentval ) const;

	/**
	 * Convert the given sum, with no currency separator switching
	 * @returns the currentval/conversion factor
	 */
	QString unconverte( const QString& currentval ) const;

	/**
	 * Convert the given sum, but don't do currency separator switching
	 * @returns the currentval/conversion factor
	 */
	QString unconverte( int currentval ) const;

	const QString& symbol() const;
	const QString& cursep() const;

 private:
	QHacc * engine;
	float factor;
	bool doconv;
	QString csymb, asymb, curs;


	private slots:
		void changedP( const QString&, float );
	void changedP( const QString&, bool );
	void changedP( const QString&, QString );
};

#endif
